/*!\file:  prognostic.cpp
 * \brief: prognostic solution
 */ 

#include "../issm.h"
#include "./parallel.h"

#undef __FUNCT__ 
#define __FUNCT__ "prognostic"

#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif


int main(int argc,char* *argv){
	
	/*I/O: */
	FILE* fid=NULL;
	char* inputfilename=NULL;
	char* outputfilename=NULL;
	char* lockname=NULL;
	char* qmuinname=NULL;
	char* qmuoutname=NULL;
	char* qmuerrname=NULL;
	int   numberofnodes;
	int waitonlock=0;

	FemModel fem;

	Vec h_g=NULL;
	Vec u_g=NULL;
	double* u_g_serial=NULL;
	double* h_g_initial=NULL;
	double* melting_g=NULL;
	double* accumulation_g=NULL;
	double  dt;
	int     qmu_analysis;

	/*Results: */
	DataSet* results=NULL;

	
	ParameterInputs* inputs=NULL;
	Param*  param=NULL;

	MODULEBOOT();

	#if !defined(_PARALLEL_) || (defined(_PARALLEL_) && !defined(_HAVE_PETSC_))
	throw ErrorException(__FUNCT__," parallel executable was compiled without support of parallel libraries!");
	#endif

	PetscInitialize(&argc,&argv,(char *)0,"");  

	/*Size and rank: */
	MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);  
	MPI_Comm_size(MPI_COMM_WORLD,&num_procs); 

	_printf_("recover , input file name and output file name:\n");
	inputfilename=argv[2];
	outputfilename=argv[3];
	lockname=argv[4];
	qmuinname=argv[5];
	qmuoutname=argv[6];
	qmuerrname=argv[7];

	/*Open handle to data on disk: */
	fid=pfopen(inputfilename,"rb");

	_printf_("read and create finite element model:\n");
	CreateFemModel(&fem,fid,"prognostic","");

	//retrieve parameters used to fill inputs
	fem.parameters->FindParam((void*)&u_g_serial,"u_g");
	fem.parameters->FindParam((void*)&h_g_initial,"h_g");
	fem.parameters->FindParam((void*)&melting_g,"m_g");
	fem.parameters->FindParam((void*)&accumulation_g,"a_g");
	fem.parameters->FindParam((void*)&dt,"dt");
	fem.parameters->FindParam((void*)&numberofnodes,"numberofnodes");

	_printf_("initialize inputs:\n");
	inputs=new ParameterInputs;
	inputs->Add("velocity",u_g_serial,3,numberofnodes);
	inputs->Add("thickness",h_g_initial,1,numberofnodes);
	inputs->Add("melting",melting_g,1,numberofnodes);
	inputs->Add("accumulation",accumulation_g,1,numberofnodes);
	inputs->Add("dt",dt);

	_printf_("initialize results:\n");
	results=new DataSet(ResultsEnum());

	/*are we running the solutoin sequence, or a qmu wrapper around it? : */
	fem.parameters->FindParam((void*)&qmu_analysis,"qmu_analysis");
	if(!qmu_analysis){

		/*run prognostic analysis: */
		_printf_("call computational core:\n");
		prognostic_core(results,&fem,inputs);

	}
	else{

		/*run qmu analysis: */
		_printf_("calling qmu analysis on prognostic core:\n");
	
		#ifdef _HAVE_DAKOTA_ 
		qmu(qmuinname,qmuoutname,qmuerrname,&fem,inputs,PrognosticAnalysisEnum(),NoneAnalysisEnum());
	 	#else
		throw ErrorException(__FUNCT__," Dakota not present, cannot do qmu!");
		#endif
	}

	_printf_("process results:\n");
	ProcessResults(&results,&fem,PrognosticAnalysisEnum());
	
	_printf_("write results to disk:\n");
	OutputResults(results,outputfilename);

	_printf_("write lock file:\n");
	fem.parameters->FindParam((void*)&waitonlock,"waitonlock");
	if (waitonlock){
		WriteLockFile(lockname);
	}
		
	_printf_("closing MPI and Petsc\n");
	PetscFinalize(); 

	/*end module: */
	MODULEEND();
	
	return 0; //unix success return;
}
